@@ -230,6 +230,49 @@ unknown_line = [
     'profile/profile_ns_ok3.sd',  # profile keyword?
     'profile/re_named_ok4.sd',  # profile keyword
     'profile/re_ok4.sd',
+
+    # multiple rules in one line
+    'bare_include_tests/ok_14.sd',
+    'bare_include_tests/ok_19.sd',
+    'bare_include_tests/ok_64.sd',
+    'bare_include_tests/ok_69.sd',
+
+    # "include if exists" and various exotic "include" variants are not supported yet
+    'bare_include_tests/ok_11.sd',
+    'bare_include_tests/ok_12.sd',
+    'bare_include_tests/ok_13.sd',
+    'bare_include_tests/ok_15.sd',
+    'bare_include_tests/ok_16.sd',
+    'bare_include_tests/ok_17.sd',
+    'bare_include_tests/ok_18.sd',
+    'bare_include_tests/ok_20.sd',
+    'bare_include_tests/ok_26.sd',
+    'bare_include_tests/ok_27.sd',
+    'bare_include_tests/ok_28.sd',
+    'bare_include_tests/ok_29.sd',
+    'bare_include_tests/ok_30.sd',
+    'bare_include_tests/ok_31.sd',
+    'bare_include_tests/ok_61.sd',
+    'bare_include_tests/ok_62.sd',
+    'bare_include_tests/ok_63.sd',
+    'bare_include_tests/ok_65.sd',
+    'bare_include_tests/ok_66.sd',
+    'bare_include_tests/ok_67.sd',
+    'bare_include_tests/ok_68.sd',
+    'bare_include_tests/ok_70.sd',
+    'bare_include_tests/ok_76.sd',
+    'bare_include_tests/ok_77.sd',
+    'bare_include_tests/ok_78.sd',
+    'bare_include_tests/ok_79.sd',
+    'bare_include_tests/ok_80.sd',
+    'bare_include_tests/ok_81.sd',
+    'bare_include_tests/ok_82.sd',
+    'bare_include_tests/ok_83.sd',
+    'bare_include_tests/ok_84.sd',
+    'bare_include_tests/ok_85.sd',
+    'bare_include_tests/ok_86.sd',
+    'bare_include_tests/ok_87.sd',
+    'bare_include_tests/ok_88.sd',
 ]
 
 # testcases with various unexpected failures
@@ -472,4 +515,4 @@ find_and_setup_test_profiles('../../parser/tst/simple_tests/')
 
 setup_all_loops(__name__)
 if __name__ == '__main__':
-    unittest.main(verbosity=1)  # reduced verbosity due to the big number of tests
+    unittest.main(verbosity=1)
